==> home-made-io-fastmag-sync-m2-0.22.2-b65669.zip: Process/Worker/ToFastmag/Integration/Customer/Data.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-17
 ******************************************************************************/

namespace Fastmag\Sync\Process\Worker\ToFastmag\Integration\Customer;

use Exception;
use Fastmag\Sync\Api\Data\Jobqueue\ToFastmagInterface as Job;
use Fastmag\Sync\Exception\ApiException;
use Fastmag\Sync\Exception\JobException;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Process\Entity\ToFastmag\Customer as CustomerEntity;
use Fastmag\Sync\Process\Worker\ToFastmag\Integration\Customer;
use Magento\Store\Model\ScopeInterface;
use stdClass;

/**
 * Class Data
 *
 * Integration class used for inserting or updating customers from Magento to Fastmag
 */
class Data extends Customer
{
    /** @var string */
    public const DEFAULT_REFERER = 'Magento';

    /** @inheritDoc */
    protected $code = 'tofastmag_integration_customer';

    /** @var string[] $subordinateWorkersAfter */
    protected $subordinateWorkersAfter = ['tofastmag_integration_customer_address'];

    /**
     * @inheritDoc
     *
     * @param Job $job
     */
    protected function processJob($job)
    {
        $entity = $this->getCustomerEntity($job);

        if ($entity->getFastmagId() === null) {
            $fastmagId = $this->getFastmagIdByEmail($entity->getEmailAddress());

            if ($fastmagId) {
                $entity->setFastmagId($fastmagId);

                $this->saveSyncCustomer($entity->getMagentoId(), $fastmagId);
            }
        }

        $newFastmagId = $this->sendDataToFastmag($entity->getMagentoId(), $entity);
        if ($newFastmagId !== false) {
            $entity->setFastmagId($newFastmagId);

            $this->saveSyncCustomer($entity->getMagentoId(), $newFastmagId);
        }
    }

    /**
     * Get customer entity from job
     *
     * @param Job $job
     *
     * @return CustomerEntity
     */
    protected function getCustomerEntity($job)
    {
        return $job->getEntity();
    }

    /**
     * Send customer to Fastmag
     *
     * @param int            $customerId
     * @param CustomerEntity $customerEntity
     *
     * @return int|bool new Fastmag ID if it's an insertion, false otherwise
     *
     * @throws JobException
     */
    protected function sendDataToFastmag($customerId, $customerEntity)
    {
        $result = false;

        if ($customerEntity->getFastmagId() === null) {
            $result = $this->createFastmagCustomer($customerId, $customerEntity);
        } else {
            $this->updateFastmagCustomer($customerId, $customerEntity);
        }

        return $result;
    }

    /**
     * Create customer on Fastmag
     *
     * @param int            $magentoId
     * @param CustomerEntity $entity
     *
     * @return int
     *
     * @throws JobException
     */
    protected function createFastmagCustomer($magentoId, $entity)
    {
        $billingAddress = $entity->getBillingAddress();

        if ($billingAddress === null) {
            throw new JobException(__(
                'Unable to create customer #%1 on Fastmag: no billing address found.',
                $entity->getMagentoId()
            ));
        }

        $customerData = $this->setDataRequest($entity);

        try {
            $response = $this->api->post('/boa/client/create/index.ips', $customerData);

            if (array_key_exists('data', $response)) {
                $response = $response['data'][0];
            }

            if ($response['status'] === 'KO') {
                $message = (array_key_exists('Message', $response) ? $response['Message'] : '');

                throw new ApiException(
                    __($message),
                    $this->api->getLastRequest(),
                    null,
                    (array_key_exists('errorCode', $response) ? $response['errorCode'] : 0)
                );
            }
        } catch (ApiException $exception) {
            throw new JobException(__(
                'Unable to create customer #%1 on Fastmag through API. Code: %2. Message: %3. Data sent: %4',
                $magentoId,
                $exception->getCode(),
                $exception->getMessage(),
                $exception->getRequest()
            ));
        }

        return (int)$response['Client'];
    }

    /**
     * Set the customer entity as intented for the API
     *
     * @param CustomerEntity $entity
     * @param bool           $update
     *
     * @return stdClass
     */
    protected function setDataRequest($entity, $update = false)
    {
        $result = [
            'Nom'           => $entity->getLastname(),
            'Prenom'        => $entity->getFirstname(),
            'Civilite'      => $entity->getPrefix(),
            'Adresse1'      => $entity->getBillingAddress()->getStreetOne(),
            'Adresse2'      => $entity->getBillingAddress()->getStreetTwo(),
            'CodePostal'    => $entity->getBillingAddress()->getPostcode(),
            'Ville'         => $entity->getBillingAddress()->getCity(),
            'Pays'          => $entity->getBillingAddress()->getCountryId(),
            'DateNaissance' => $entity->getDob(),
            'Telephone'     => $entity->getBillingAddress()->getPhoneNumber(),
            'Email'         => $entity->getEmailAddress(),
            'Provenance'    => self::DEFAULT_REFERER,
            'Societe'       => $entity->getBillingAddress()->getCompany(),
            'CodeExterne'   => $entity->getMagentoId(),
            'Sexe'          => $entity->getGender(),
            'PubParEMail'   => ($entity->getSubscribeNewsletter() ? '1' : '0')
        ];

        if ($update) {
            $result['Client'] = $entity->getFastmagId();
        } else {
            $result['Magasin'] = $this->config->getValue(
                Config::XML_PATH_ORDER_WORKFLOW_SHOP,
                ScopeInterface::SCOPE_STORES,
                $entity->getStoreId()
            );
        }

        foreach ($result as $field => $data) {
            if ($data === null || $data === '') {
                unset($result[$field]);
            }
        }

        return (object)['Clients' => [(object)$result]];
    }

    /**
     * Update customer on Fastmag
     *
     * @param int            $magentoId
     * @param CustomerEntity $entity
     *
     * @return void
     *
     * @throws JobException
     */
    protected function updateFastmagCustomer($magentoId, $entity)
    {
        $this->checkFastmagEmail($entity->getFastmagId(), $entity->getEmailAddress());

        $billingAddress = $entity->getBillingAddress();

        if ($billingAddress === null) {
            throw new JobException(__(
                'Unable to create customer #%1 on Fastmag: no billing address found.',
                $entity->getMagentoId()
            ));
        }

        $customerData = $this->setDataRequest($entity, true);

        try {
            $response = $this->api->post('/boa/client/update/index.ips', $customerData);

            if (array_key_exists('data', $response)) {
                $response = $response['data'][0];
            }

            if ($response['status'] === 'KO') {
                $message = (array_key_exists('Message', $response) ? $response['Message'] : '');

                throw new ApiException(
                    __($message),
                    $this->api->getLastRequest(),
                    null,
                    (array_key_exists('errorCode', $response) ? $response['errorCode'] : 0)
                );
            }
        } catch (ApiException $exception) {
            throw new JobException(__(
                'Unable to update customer #%1 on Fastmag through API. Code: %2. Message: %3. Data sent: %4',
                $magentoId,
                $exception->getCode(),
                $exception->getMessage(),
                $exception->getRequest()
            ));
        }
    }

    /**
     * Check email of the customer on Fastmag, and change it if it's not the same as the one in Magento
     *
     * @param int    $fastmagId
     * @param string $email
     *
     * @return void
     *
     * @throws JobException
     */
    protected function checkFastmagEmail($fastmagId, $email)
    {
        try {
            $sql = 'SELECT Email AS email
                FROM clients
                WHERE Client = ' . $this->getFastmagSqlConnection()->escape($fastmagId);

            $rows = $this->getFastmagSqlConnection()->get($sql);

            if ($rows[0]['email'] !== $email) {
                $this->changeCustomerEmail($fastmagId, $email);
            }
        } catch (Exception $exception) {
            throw new JobException(__(
                'Error when trying to check customer email on Fastmag. Message: %1. Customers IDs: %2',
                $exception->getMessage(),
                $fastmagId
            ));
        }
    }

    /**
     * Change customer email on Fastmag
     *
     * @param string $oldEmail
     * @param string $newEmail
     *
     * @return void
     *
     * @throws JobException
     */
    protected function changeCustomerEmail($oldEmail, $newEmail)
    {
        $data = (object)['Emails' => [(object)['AncienMail'  => $oldEmail, 'NouveauMail' => $newEmail]]];

        try {
            $response = $this->api->post('/boa/client/email/index.ips', $data);

            if ($response['status'] === 'KO') {
                $message = (array_key_exists('Message', $response) ? $response['Message'] : '');

                throw new ApiException(
                    __($message),
                    $this->api->getLastRequest(),
                    null,
                    (array_key_exists('errorCode', $response) ? $response['errorCode'] : 0)
                );
            }
        } catch (ApiException $exception) {
            throw new JobException(__(
                'Unable to update the email "%1" on Fastmag through API. Code: %2. Message: %3. Data sent: %3',
                $oldEmail,
                $exception->getCode(),
                $exception->getMessage(),
                $exception->getRequest()
            ));
        }
    }
}

==> home-made-io-fastmag-sync-m2-0.24.0-762a73.zip: Process/Worker/ToFastmag/Integration/Customer/Data.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-29
 ******************************************************************************/

namespace Fastmag\Sync\Process\Worker\ToFastmag\Integration\Customer;

use Exception;
use Fastmag\Sync\Api\Data\Jobqueue\ToFastmagInterface as Job;
use Fastmag\Sync\Exception\ApiException;
use Fastmag\Sync\Exception\JobException;
use Fastmag\Sync\Exception\NoBillingAddressException;
use Fastmag\Sync\Exception\ProcessException;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Process\Entity\ToFastmag\Customer as CustomerEntity;
use Fastmag\Sync\Process\Worker\ToFastmag\Integration\Customer;
use Magento\Store\Model\ScopeInterface;
use stdClass;

/**
 * Class Data
 *
 * Integration class used for inserting or updating customers from Magento to Fastmag
 */
class Data extends Customer
{
    /** @var string */
    public const DEFAULT_REFERER = 'Magento';

    /** @inheritDoc */
    protected $code = 'tofastmag_integration_customer';

    /** @var string $hydrationWorker */
    protected $hydrationWorker = 'tofastmag_hydration_customer';

    /** @var string[] $subordinateWorkersAfter */
    protected $subordinateWorkersAfter = ['tofastmag_integration_customer_address'];

    /**
     * @inheritDoc
     */
    public function run()
    {
        try {
            $this->initiate();
        } catch (ProcessException $exception) {
            $this->logger->notice($exception->getMessage());

            return;
        }

        foreach ($this->getJobs()->getItems() as $job) {
            if (!$job->isInError()) {
                try {
                    $this->processJob($job);
                } catch (JobException $exception) {
                    $this->invalidateJob($job, $exception);
                } catch (NoBillingAddressException $exception) {
                    $this->skipJob($job, $exception);
                }

                $this->saveJob($job);
            }
        }
    }

    /**
     * @inheritDoc
     */
    public function isEnabled()
    {
        return $this->config->isSetFlag(Config::XML_PATH_CUSTOMER_EXPORT_ENABLE);
    }

    /**
     * @inheritDoc
     *
     * @param Job $job
     *
     * @throws NoBillingAddressException
     */
    protected function processJob($job)
    {
        $entity = $this->getCustomerEntity($job);

        $billingAddress = $entity->getBillingAddress();
        if ($billingAddress === null) {
            throw new NoBillingAddressException(__(
                'Unable to create customer #%1 on Fastmag: no billing address found.',
                $entity->getMagentoId()
            ));
        }

        if ($entity->getFastmagId() === null) {
            $fastmagId = $this->getFastmagIdByEmail($entity->getEmailAddress());

            if ($fastmagId) {
                $entity->setFastmagId($fastmagId);

                $this->saveSyncCustomer($entity->getMagentoId(), $fastmagId);
            }
        }

        $newFastmagId = $this->sendDataToFastmag($entity->getMagentoId(), $entity);
        if ($newFastmagId !== false) {
            $entity->setFastmagId($newFastmagId);

            $this->saveSyncCustomer($entity->getMagentoId(), $newFastmagId);
        }
    }

    /**
     * Get customer entity from job
     *
     * @param Job $job
     *
     * @return CustomerEntity
     */
    protected function getCustomerEntity($job)
    {
        return $job->getEntity();
    }

    /**
     * Send customer to Fastmag
     *
     * @param int            $customerId
     * @param CustomerEntity $customerEntity
     *
     * @return int|bool new Fastmag ID if it's an insertion, false otherwise
     *
     * @throws JobException
     */
    protected function sendDataToFastmag($customerId, $customerEntity)
    {
        $result = false;

        if ($customerEntity->getFastmagId() === null) {
            $result = $this->createFastmagCustomer($customerId, $customerEntity);
        } else {
            $this->updateFastmagCustomer($customerId, $customerEntity);
        }

        return $result;
    }

    /**
     * Create customer on Fastmag
     *
     * @param int            $magentoId
     * @param CustomerEntity $entity
     *
     * @return int
     *
     * @throws JobException
     */
    protected function createFastmagCustomer($magentoId, $entity)
    {
        $customerData = $this->setDataRequest($entity);

        try {
            $response = $this->api->post('/boa/client/create/index.ips', $customerData);

            if (array_key_exists('data', $response)) {
                $response = $response['data'][0];
            }

            if ($response['status'] === 'KO') {
                $message = (array_key_exists('Message', $response) ? $response['Message'] : '');

                throw new ApiException(
                    __($message),
                    $this->api->getLastRequest(),
                    null,
                    (array_key_exists('errorCode', $response) ? $response['errorCode'] : 0)
                );
            }
        } catch (ApiException $exception) {
            throw new JobException(__(
                'Unable to create customer #%1 on Fastmag through API. Code: %2. Message: %3. Data sent: %4',
                $magentoId,
                $exception->getCode(),
                $exception->getMessage(),
                $exception->getRequest()
            ));
        }

        return (int)$response['Client'];
    }

    /**
     * Set the customer entity as intented for the API
     *
     * @param CustomerEntity $entity
     * @param bool           $update
     *
     * @return stdClass
     */
    protected function setDataRequest($entity, $update = false)
    {
        $result = [
            'Nom'           => $entity->getLastname(),
            'Prenom'        => $entity->getFirstname(),
            'Civilite'      => $entity->getPrefix(),
            'Adresse1'      => $entity->getBillingAddress()->getStreetOne(),
            'Adresse2'      => $entity->getBillingAddress()->getStreetTwo(),
            'CodePostal'    => $entity->getBillingAddress()->getPostcode(),
            'Ville'         => $entity->getBillingAddress()->getCity(),
            'Pays'          => $entity->getBillingAddress()->getCountryId(),
            'DateNaissance' => $entity->getDob(),
            'Telephone'     => $entity->getBillingAddress()->getPhoneNumber(),
            'Email'         => $entity->getEmailAddress(),
            'Provenance'    => self::DEFAULT_REFERER,
            'Societe'       => $entity->getBillingAddress()->getCompany(),
            'CodeExterne'   => $entity->getMagentoId(),
            'Sexe'          => $entity->getGender(),
            'PubParEMail'   => ($entity->getSubscribeNewsletter() ? '1' : '0')
        ];

        if ($update) {
            $result['Client'] = $entity->getFastmagId();
        } else {
            $result['Magasin'] = $this->config->getValue(
                Config::XML_PATH_ORDER_WORKFLOW_SHOP,
                ScopeInterface::SCOPE_STORES,
                $entity->getStoreId()
            );
        }

        foreach ($result as $field => $data) {
            if ($data === null || $data === '') {
                unset($result[$field]);
            }
        }

        return (object)['Clients' => [(object)$result]];
    }

    /**
     * Update customer on Fastmag
     *
     * @param int            $magentoId
     * @param CustomerEntity $entity
     *
     * @return void
     *
     * @throws JobException
     */
    protected function updateFastmagCustomer($magentoId, $entity)
    {
        $this->checkFastmagEmail($entity->getFastmagId(), $entity->getEmailAddress());

        $customerData = $this->setDataRequest($entity, true);

        try {
            $response = $this->api->post('/boa/client/update/index.ips', $customerData);

            if (array_key_exists('data', $response)) {
                $response = $response['data'][0];
            }

            if ($response['status'] === 'KO') {
                $message = (array_key_exists('Message', $response) ? $response['Message'] : '');

                throw new ApiException(
                    __($message),
                    $this->api->getLastRequest(),
                    null,
                    (array_key_exists('errorCode', $response) ? $response['errorCode'] : 0)
                );
            }
        } catch (ApiException $exception) {
            throw new JobException(__(
                'Unable to update customer #%1 on Fastmag through API. Code: %2. Message: %3. Data sent: %4',
                $magentoId,
                $exception->getCode(),
                $exception->getMessage(),
                $exception->getRequest()
            ));
        }
    }

    /**
     * Check email of the customer on Fastmag, and change it if it's not the same as the one in Magento
     *
     * @param int    $fastmagId
     * @param string $email
     *
     * @return void
     *
     * @throws JobException
     */
    protected function checkFastmagEmail($fastmagId, $email)
    {
        try {
            $sql = 'SELECT Email AS email
                FROM clients
                WHERE Client = ' . $this->getFastmagSqlConnection()->escape($fastmagId);

            $rows = $this->getFastmagSqlConnection()->get($sql);

            if ($rows[0]['email'] !== $email) {
                $this->changeCustomerEmail($fastmagId, $email);
            }
        } catch (Exception $exception) {
            throw new JobException(__(
                'Error when trying to check customer email on Fastmag. Message: %1. Customers IDs: %2',
                $exception->getMessage(),
                $fastmagId
            ));
        }
    }

    /**
     * Change customer email on Fastmag
     *
     * @param string $oldEmail
     * @param string $newEmail
     *
     * @return void
     *
     * @throws JobException
     */
    protected function changeCustomerEmail($oldEmail, $newEmail)
    {
        $data = (object)['Emails' => [(object)['AncienMail'  => $oldEmail, 'NouveauMail' => $newEmail]]];

        try {
            $response = $this->api->post('/boa/client/email/index.ips', $data);

            if ($response['status'] === 'KO') {
                $message = (array_key_exists('Message', $response) ? $response['Message'] : '');

                throw new ApiException(
                    __($message),
                    $this->api->getLastRequest(),
                    null,
                    (array_key_exists('errorCode', $response) ? $response['errorCode'] : 0)
                );
            }
        } catch (ApiException $exception) {
            throw new JobException(__(
                'Unable to update the email "%1" on Fastmag through API. Code: %2. Message: %3. Data sent: %3',
                $oldEmail,
                $exception->getCode(),
                $exception->getMessage(),
                $exception->getRequest()
            ));
        }
    }
}
